# -*- coding: utf-8 -*-

# django-messages-to-bootstrap-notify
# dm2bn/apps.py

from __future__ import unicode_literals

from django.apps import AppConfig


__all__ = [
    "Config",
]


class Config(AppConfig):

    name = "dm2bn"
    verbose_name = "Django messages to bootstrap-notify"
